<?php

namespace Drupal\block_class\Service;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Database\Connection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Component\Utility\Html;
use Drupal\block_class\Constants\BlockClassConstants;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Session\AccountProxy;

/**
 * Block Class Service Class.
 */
class BlockClassHelperService {

  use StringTranslationTrait;

  /**
   * The UUID service.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidService;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Path Matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * The current path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * Drupal\Core\Entity\EntityRepositoryInterface service.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Drupal\Core\Session\AccountProxy definition.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The block entity.
   *
   * @var useDrupal\block\Entity\Block
   */
  protected $blockEntity;

  /**
   * Construct of Block Class service.
   */
  public function __construct(LanguageManagerInterface $language_manager, ConfigFactoryInterface $config_factory, Connection $database, RequestStack $request_stack, PathMatcherInterface $path_matcher, UuidInterface $uuid_service, AliasManagerInterface $alias_manager, CurrentPathStack $current_path, EntityRepositoryInterface $entityRepository, AccountProxy $currentUser, EntityTypeManagerInterface $entity_manager) {
    $this->languageManager = $language_manager;
    $this->pathMatcher = $path_matcher;
    $this->request = $request_stack->getCurrentRequest();
    $this->configFactory = $config_factory;
    $this->database = $database;
    $this->uuidService = $uuid_service;
    $this->aliasManager = $alias_manager;
    $this->currentPath = $current_path;
    $this->entityRepository = $entityRepository;
    $this->currentUser = $currentUser;
    $this->entityTypeManager = $entity_manager;
    $this->blockEntity = $this->entityTypeManager->getStorage('block');
  }

  /**
   * Method to do the presave block.
   */
  public function blockClassPreSave(&$entity) {

    // If there is no class, unset the Third Party Setting.
    if (empty($entity->getThirdPartySetting('block_class', 'classes'))) {
      $entity->unsetThirdPartySetting('block_class', 'classes');
    }

    // If there is no attributes, unset the Third Party Setting.
    if (empty($entity->getThirdPartySetting('block_class', 'attributes'))) {
      $entity->unsetThirdPartySetting('block_class', 'attributes');
    }

    // Get the config object.
    $config = $this->configFactory->getEditable('block_class.settings');

    // Get the default case on settings.
    $default_case = $config->get('default_case', 'standard');

    // Get the block class.
    $block_classes = $entity->getThirdPartySetting('block_class', 'classes');

    switch ($default_case) {

      case 'uppercase':

        $block_classes = strtoupper($block_classes);

        break;

      case 'lowercase':

        $block_classes = strtolower($block_classes);

        break;

    }

    // Set the Third Party Settings.
    $entity->setThirdPartySetting('block_class', 'classes', $block_classes);

    // Get the config object.
    $config = $this->configFactory->getEditable('block_class.settings');

    // Get the current classes stored.
    $block_classes_stored = $config->get('block_classes_stored');

    // Get the array from JSON.
    $block_classes_stored = Json::decode($block_classes_stored);

    // Verify if is empty.
    if (empty($block_classes_stored)) {
      $block_classes_stored = [];
    }

    // Get the current class and export to array.
    $current_block_classes = explode(' ', $block_classes);

    // Use the key the same as value.
    $current_block_classes = array_combine($current_block_classes, $current_block_classes);

    // Merge with the current one.
    $block_classes_to_store = array_merge($current_block_classes, $current_block_classes);

    // Get as JSON.
    $block_classes_to_store = Json::encode($block_classes_to_store);

    // Store in the config.
    $config->set('block_classes_stored', $block_classes_to_store);

    // Save.
    $config->save();

  }

  /**
   * Method to redirect to Bulk Operations.
   */
  public function redirectToBulkOperations() {

    // Get path bulk operation.
    $bulk_operation_path = Url::fromRoute('block_class.bulk_operations')->toString();

    // Get response.
    $response = new RedirectResponse($bulk_operation_path);

    // Send to confirmation.
    $response->send();
    exit;

  }

  /**
   * Method to do the preprocess block.
   */
  public function blockClassPreprocessBlock(&$variables) {

    // Blocks coming from page manager widget does not have id. If there is no
    // Block ID, skip that.
    if (empty($variables['elements']['#id'])) {
      return;
    }

    // If there is no block with this ID, skip.
    if (empty($this->blockEntity->load($variables['elements']['#id']))) {
      return;
    }

    // Load the block by ID.
    $block = $this->blockEntity->load($variables['elements']['#id']);

    // Add attributes on block.
    $this->addAttributesOnBlock($block, $variables);

    // Add classes on block.
    $this->addClassesOnBlock($block, $variables);

  }

  /**
   * Method to add attributes on block.
   */
  public function addAttributesOnBlock(&$block, &$variables) {

    // Get the config object.
    $config = $this->configFactory->getEditable('block_class.settings');

    // If attributes isn't enabled, skip.
    if (empty($config->get('enable_attributes'))) {
      return FALSE;
    }

    // If there is no attributes on block, skip.
    if (empty($block->getThirdPartySetting('block_class', 'attributes'))) {
      return FALSE;
    }

    $attributes = $block->getThirdPartySetting('block_class', 'attributes');

    $attributes = explode(PHP_EOL, $attributes);

    foreach ($attributes as $attribute) {

      $attribute = explode('|', $attribute);

      $attribute_key = trim($attribute[0]);

      $attribute_value = trim($attribute[1]);

      // Insert the attributes.
      $variables['attributes'][$attribute_key][] = $attribute_value;
    }
  }

  /**
   * Method to add classes on block.
   */
  public function addClassesOnBlock(&$block, &$variables) {

    // Verify if the current block has Third Party Settings with classes.
    $classes = $block->getThirdPartySetting('block_class', 'classes');

    if (empty($classes)) {
      return FALSE;
    }

    // Get all classes if exists.
    $classes_array = explode(' ', $classes);

    // Get the config object.
    $config = $this->configFactory->getEditable('block_class.settings');

    $filter_css_identifier = [];

    if (!empty($config->get('filter_html_clean_css_identifier'))) {

      $filter_clean_css_identifier = $config->get('filter_html_clean_css_identifier');

      $filter_clean_css_identifier = explode(PHP_EOL, $filter_clean_css_identifier);

      foreach ($filter_clean_css_identifier as $filter) {

        $filter = explode('|', $filter);

        $origin_char = trim($filter[0]);
        $target_char = trim($filter[1]);

        $filter_css_identifier[$origin_char] = $target_char;

      }

    }

    // Add all classes.
    foreach ($classes_array as $class) {
      $variables['attributes']['class'][] = Html::cleanCssIdentifier($class, $filter_css_identifier);
    }
  }

  /**
   * Method to do a form alter.
   */
  public function blockClassFormAlter(&$form, &$form_state) {

    // If the user don't have permission, skip that.
    if (!$this->currentUser->hasPermission(BlockClassConstants::BLOCK_CLASS_PERMISSION)) {
      return;
    }

    $form_object = $form_state->getFormObject();

    if ($form_object instanceof EntityFormInterface) {

      $qty_classes_per_block = 10;

      // Get config object.
      $config = $this->configFactory->getEditable('block_class.settings');

      if (!empty($config->get('qty_classes_per_block'))) {
        $qty_classes_per_block = $config->get('qty_classes_per_block');
      }

      // Get the URL settings global page.
      $url_settings_page = Url::fromRoute('block_class.settings')->toString();

      $url_used_items_list = Url::fromRoute('block_class.class_list')->toString();

      // Put the default help text.
      $help_text = $this->t('Customize the styling of this block by adding CSS classes.');

      // Put the Modal with all items used.
      $help_text .= ' ' . $this->t('<a href="@url_used_items_list@" class="use-ajax" data-dialog-options="{&quot;width&quot;:800}" data-dialog-type="modal">Click here</a> to see all the classes used.', [
        '@url_used_items_list@' => $url_used_items_list,
      ]);

      $form['class'] = [
        '#type' => 'details',
        '#title' => $this->t('Class'),
        '#open' => TRUE,
        '#description' => $help_text,
      ];

      /** @var \Drupal\block\BlockInterface $block */
      $block = $form_object->getEntity();

      // This will automatically be saved in the third party settings.
      $form['class']['third_party_settings']['#tree'] = TRUE;

      // Default field type.
      $field_type = 'textfield';

      // Default value for maxlength.
      $maxlength_block_class_field = 255;

      // Get the field type if exists.
      if (!empty($config->get('field_type')) && $config->get('field_type') != 'multiple_textfields') {
        $field_type = $config->get('field_type');
      }

      // Get maxlength if exists.
      if (!empty($config->get('maxlength_block_class_field'))) {
        $maxlength_block_class_field = $config->get('maxlength_block_class_field');
      }

      $image_path = '/' . drupal_get_path('module', 'block_class') . '/images/';

      if ($config->get('field_type') == 'textfield') {

        // Remove the help text in the field group because the field will have
        // their help text.
        unset($form['class']['#description']);

        $form['class']['third_party_settings']['block_class']['classes'] = [
          '#type' => $field_type,
          '#title' => $this->t('CSS class(es)'),
          '#description' => $this->t('Customize the styling of this block by adding CSS classes. Separate multiple classes by spaces. The maxlength configured is @maxlength_block_class_field@. If necessary you can update it <a href="/admin/config/content/block-class/settings">here</a>. This class will appear in the first level of block. <a href="@image_path@/example-1.png">Click here</a> to see a example', [
            '@maxlength_block_class_field@' => $maxlength_block_class_field,
            '@image_path@' => $image_path,
          ]),
          '#default_value' => $block->getThirdPartySetting('block_class', 'classes'),
          '#maxlength' => $maxlength_block_class_field,
        ];

      }

      if ($config->get('field_type') == 'multiple_textfields') {

        // Get the classes on getThirdPartySettings.
        $classes = $block->getThirdPartySetting('block_class', 'classes');

        // Explode by spaces to get all classes in an array.
        $classes = explode(' ', $classes);

        // If the quantity of items in the block is lower than the quantity of
        // items configured in the settings we need to have the higher limit
        // just to avoid some items being cut due the limit.
        if ((int) $qty_classes_per_block < (int) count($classes)) {
          $qty_classes_per_block = (int) count($classes);
        }

        // Run a for to add 10 multiple fields.
        for ($index = 0; $index <= ($qty_classes_per_block - 1); $index++) {

          // Initial value.
          $multiclass_default_value = '';

          // Verify if there is a value on this class to add in the field.
          if (!empty($classes[$index])) {
            $multiclass_default_value = $classes[$index];
          }

          // Insert the new field.
          $form['class']['third_party_settings']['block_class']['classes_' . $index] = [
            '#type' => 'textfield',
            '#title' => $this->t('CSS class'),
            '#default_value' => $multiclass_default_value,
            '#maxlength' => $maxlength_block_class_field,
          ];

          // Enable the auto-complete only is selected in the settings page.
          if (!empty($config->get('enable_auto_complete'))) {
            $form['class']['third_party_settings']['block_class']['classes_' . $index]['#autocomplete_route_name'] = 'block_class.autocomplete';
          }

          // Insert a default class for all classes visible or not.
          $form['class']['third_party_settings']['block_class']['classes_' . $index]['#attributes']['class'][] = 'multiple-textfield';

          // Put the visible class by default.
          $form['class']['third_party_settings']['block_class']['classes_' . $index]['#attributes']['class'][] = 'displayed-class-field';

          // If is the second o higher and there is no class for that, hide.
          if ($index >= 1 && empty($classes[$index])) {

            $form['class']['third_party_settings']['block_class']['classes_' . $index]['#attributes']['class'][] = 'hidden-class-field';

            // If this class should be hidden, get the key to put the right
            // class on that.
            $hidden_class_key = array_search('displayed-class-field', $form['class']['third_party_settings']['block_class']['classes_' . $index]['#attributes']['class']);

            // Unset in the array to remove the hidden class.
            unset($form['class']['third_party_settings']['block_class']['classes_' . $index]['#attributes']['class'][$hidden_class_key]);

          }

        }

        // Add another item button in the last field.
        $form['class']['third_party_settings']['block_class']['add_another_item'] = [
          '#type' => 'button',
          '#value' => 'Add another class',
        ];

        // Add the class to identity the "add another item" button.
        $form['class']['third_party_settings']['block_class']['add_another_item']['#attributes']['class'][] = 'block-class-add-another-item';

        // Add remove item button in the last field.
        $form['class']['third_party_settings']['block_class']['remove_item'] = [
          '#type' => 'button',
          '#value' => 'Remove class',
        ];

        // Add the class to identity the "Remove item" button.
        $form['class']['third_party_settings']['block_class']['remove_item']['#attributes']['class'][] = 'block-class-remove-item';

        // Verify if there is a help text for qty item and if not set a default
        // value for this.
        if (empty($help_text_qty_items)) {
          $help_text_qty_items = '';
        }

        $help_text_qty_items .= ' ' . $this->t('The maximum of classes per block is @qty_classes_per_block@ but if you need you can update this value in the <a href="@url_settings_page@">settings page</a>', [
          '@qty_classes_per_block@' => $qty_classes_per_block,
          '@url_settings_page@' => $url_settings_page,
        ]);

        $form['class']['third_party_settings']['block_class']['help_text_qty_items'] = [
          '#type' => 'markup',
          '#markup' => '<p class="help-text-qty-items help-text-qty-items-hidden">' . $help_text_qty_items . '</p>',
        ];
      }

      $form['class']['third_party_settings']['block_class']['classes']['#attributes']['class'][] = 'block-class-class';

      if (!empty($config->get('enable_attributes'))) {

        $attributes = $block->getThirdPartySetting('block_class', 'attributes');

        $attributes = explode(PHP_EOL, $attributes);

        $qty_attributes_per_block = 10;

        if (!empty($config->get('qty_attributes_per_block'))) {
          $qty_attributes_per_block = $config->get('qty_attributes_per_block');
        }

        // If the quantity of items in the block is lower than the quantity of
        // items configured in the settings we need to have the higher limit
        // just to avoid some items being cut due the limit.
        if ((int) $qty_attributes_per_block < (int) count($attributes)) {
          $qty_attributes_per_block = (int) count($attributes);
        }

        // Add the attributes dynamically based on the settings field of global
        // settings page.
        for ($index = 0; $index <= ($qty_attributes_per_block - 1); $index++) {

          // Default value for attribute key and attribute value.
          $attribute_key = '';
          $attribute_value = '';

          // Verify if there an attribute to be filled.
          if (!empty($attributes[$index])) {

            $attribute = explode('|', $attributes[$index]);

            // Get the attribute key and attribute value.
            $attribute_key = $attribute[0];
            $attribute_value = $attribute[1];

          }

          // Create the help text for the multiple attribute fields.
          $help_text = $this->t('Customize the this block by adding attributes. E.g. "data-block-type"="admin"');

          // Create the field group for multiple attributes.
          $form['multiple_attributes']['third_party_settings']['block_class']['attribute_' . $index] = [
            '#type' => 'details',
            '#title' => $this->t('Attribute'),
            '#open' => TRUE,
            '#description' => $help_text,
          ];

          // Default value for maxlength.
          $maxlength_multiple_attributes = 255;

          // Get maxlength if exists.
          if (!empty($config->get('maxlength_attributes'))) {
            $maxlength_multiple_attributes = $config->get('maxlength_attributes');
          }

          $form['multiple_attributes']['third_party_settings']['block_class']['attribute_' . $index]['attribute_key_' . $index] = [
            '#type' => 'textfield',
            '#title' => $this->t('Attribute Key'),
            '#default_value' => $attribute_key,
            '#description' => $this->t('Set the attribute key. E.g. data-block-type'),
            '#maxlength' => $maxlength_multiple_attributes,
          ];

          $form['multiple_attributes']['third_party_settings']['block_class']['attribute_' . $index]['attribute_value_' . $index] = [
            '#type' => 'textfield',
            '#title' => $this->t('Attribute Value'),
            '#default_value' => $attribute_value,
            '#description' => $this->t('Set the attribute value. E.g. admin'),
            '#maxlength' => $maxlength_multiple_attributes,
          ];

          $form['multiple_attributes']['third_party_settings']['block_class']['attribute_' . $index]['#attributes']['class'][] = 'multiple-textfield-attribute';

          if ($index == 0 || (!empty($attribute_key) && !empty($attribute_value))) {
            $form['multiple_attributes']['third_party_settings']['block_class']['attribute_' . $index]['#attributes']['class'][] = 'displayed-attribute-field';
          }
          else {
            $form['multiple_attributes']['third_party_settings']['block_class']['attribute_' . $index]['#attributes']['class'][] = 'hidden-attribute-field';
          }
        }

        $form['multiple_attributes']['add_another_attribute'] = [
          '#type' => 'button',
          '#value' => 'Add another attribute',
        ];

        $form['multiple_attributes']['add_another_attribute']['#attributes']['class'][] = 'block-class-add-another-attribute';

        $form['multiple_attributes']['remove_attribute'] = [
          '#type' => 'button',
          '#value' => 'Remove attribute',
        ];

        $form['multiple_attributes']['remove_attribute']['#attributes']['class'][] = 'block-class-remove-attribute';

      }

      if (empty($block->getThirdPartySetting('block_class', 'classes')) && !empty($config->get('enable_auto_complete'))) {
        $form['third_party_settings']['block_class']['classes']['#autocomplete_route_name'] = 'block_class.autocomplete';
      }
    }

    $form['#validate'][] = 'block_class_form_block_form_validate';

    $form['#attached']['library'][] = 'block_class/block-class';

    // Get the default case on settings.
    $default_case = $config->get('default_case', 'standard');

    // Put the "default case" in the Drupal settings to be used in the JS.
    $form['#attached']['drupalSettings']['block_class']['default_case'] = $default_case;

  }

  /**
   * Method to do a form validate.
   */
  public function blockClassFormValidate(&$form, &$form_state) {

    // Get the config object.
    $config = $this->configFactory->getEditable('block_class.settings');

    // Validate dynamic items.
    $this->validateDynamicClasses($form, $form_state, $config);

    // Validate dynamic attributes.
    $this->validateAttributes($form, $form_state, $config);

    // Validate class.
    $this->validateClass($form, $form_state, $config);
  }

  /**
   * Method to validate Dynamic items.
   */
  public function validateDynamicClasses(&$form, &$form_state, $config) {

    // Get the ThirdPartySettings.
    $third_party_settings = $form_state->getValue('class')['third_party_settings'];

    // Verify if there is attributes enabled.
    if (!empty($form_state->getValue('attributes')['third_party_settings'])) {
      $third_party_settings['block_class']['attributes'] = $form_state->getValue('attributes')['third_party_settings']['block_class']['attributes'];
    }

    // Remove unused items.
    unset($third_party_settings['block_class']['add_another_item']);
    unset($third_party_settings['block_class']['remove_item']);

    // Clear empty values.
    $third_party_settings['block_class'] = array_filter($third_party_settings['block_class']);

    // Set the ThirdPartySettings with the default array.
    $form_state->setValue('third_party_settings', $third_party_settings);

    // Unset the old values.
    $form_state->unsetValue('class');

    // Unset the old values for attributes.
    $form_state->unsetValue('attributes');

    if ($config->get('field_type') == 'multiple_textfields') {

      // Initial value for class.
      $classes = '';

      $third_party_settings = $form_state->getValue('third_party_settings');

      $classes_field = $third_party_settings['block_class'];

      // Unset values that aren't classes.
      unset($classes_field['attributes']);
      unset($classes_field['remove_item']);
      unset($classes_field['add_another_item']);

      // Removed blank values.
      $classes_field = array_filter($classes_field);

      // If there are duplicated classes send a message.
      if (!empty(count($classes_field) !== count(array_unique($classes_field)))) {
        $form_state->setErrorByName('class][third_party_settings][block_class', $this->t("There are duplicated classes"));
        return FALSE;
      }

      foreach ($classes_field as $field_id => $class_field) {

        if ($field_id == 'classes' || $field_id == 'attributes' || $field_id == 'remove_item' || $field_id == 'add_another_item') {
          continue;
        }

        // Verify in the backend if there is space in the block class field and
        // Send a message that isn't necessary they're using multiple class.
        if (strpos($class_field, ' ')) {
          $form_state->setErrorByName('class][third_party_settings][block_class][' . $field_id, $this->t("Spaces isn't necessary since you're using multiple class field. Use one class per field"));
          return FALSE;
        }

        // Validate if the first char is a point and say that isn't necessary.
        if (strpos($class_field, '.') === 0) {
          $form_state->setErrorByName('class][third_party_settings][block_class][' . $field_id, $this->t("Isn't necessary add point in the beginning of class"));
          return FALSE;
        }

        // Concatenate the classes.
        $classes .= ' ' . $class_field;

      }

      $classes = trim($classes);

      $third_party_settings['block_class']['classes'] = $classes;

      // Remove unused values from Third Party Settings.
      foreach ($third_party_settings['block_class'] as $key => $third_party_setting) {

        // If there is a classes_ we can remove.
        if (str_contains($key, 'classes_')) {
          unset($third_party_settings['block_class'][$key]);
        }

      }

      $form_state->setValue('third_party_settings', $third_party_settings);

    }
  }

  /**
   * Method to validate Dynamic attributes.
   */
  public function validateAttributes(&$form, &$form_state, $config) {

    if (empty($form_state->getValue('multiple_attributes')['third_party_settings'])) {
      return FALSE;
    }

    $third_party_settings = $form_state->getValue('third_party_settings');

    $third_party_settings['block_class']['multiple_attributes'] = $form_state->getValue('multiple_attributes')['third_party_settings']['block_class'];

    // Set the ThirdPartySettings with the default array.
    $form_state->setValue('third_party_settings', $third_party_settings);

    // Unset the old values.
    $form_state->unsetValue('multiple_attributes');

    // Initial value for attributes.
    $attributes = '';

    $multiple_attributes = $third_party_settings['block_class']['multiple_attributes'];

    $index = 0;

    $attributes = '';

    $attribute_keys = [];

    foreach ($multiple_attributes as $key => $attribute) {

      // If both key and attribute are empty, skip that.
      if (empty($attribute['attribute_key_' . $index]) && empty($attribute['attribute_value_' . $index])) {
        unset($multiple_attributes[$key]);
        $index++;
        continue;
      }

      $attribute_key = $attribute['attribute_key_' . $index];

      $attribute_value = $attribute['attribute_value_' . $index];

      // Search by duplicated attributes.
      $attribute_already_exists = array_search($attribute_key, $attribute_keys);

      // Validate if this attribute already exists.
      if ($attribute_already_exists !== FALSE) {

        $form_state->setErrorByName('multiple_attributes][third_party_settings][block_class][attribute_' . $index . '][attribute_key_' . $index, $this->t('There are duplicated classes'));

        return FALSE;

      }

      // Attributr keys to validated with duplicated.
      $attribute_keys[] = $attribute_key;

      if (empty($attribute_key)) {

        $form_state->setErrorByName('multiple_attributes][third_party_settings][block_class][attribute_' . $index . '][attribute_key_' . $index, $this->t('You need to fill the attribute key'));

        return FALSE;
      }

      if (empty($attribute_value)) {

        $form_state->setErrorByName('multiple_attributes][third_party_settings][block_class][attribute_' . $index . '][attribute_value_' . $index, $this->t('You need to fill the attribute value'));

        return FALSE;
      }

      // Validate ID attribute.
      if ($attribute_key == 'id') {

        // Set a message informing that we can't update the ID attribute.
        $form_state->setErrorByName('multiple_attributes][third_party_settings][block_class][attribute_' . $index . '][attribute_key_' . $index, $this->t("You can't the attribute id"));

        // Return False on validation.
        return FALSE;

      }

      // Validate class attribute.
      if ($attribute_key == 'class') {

        // Set a message informing that we can't use the class attribute.
        $form_state->setErrorByName('multiple_attributes][third_party_settings][block_class][attribute_' . $index . '][attribute_key_' . $index, $this->t("You can't use class. Use the field class instead"));

        // Return False on validation.
        return FALSE;

      }

      $attributes .= PHP_EOL . $attribute['attribute_key_' . $index] . '|' . $attribute['attribute_value_' . $index];

      // If there is a settings to allow only letters and numbers, validate it.
      if (!empty($config->get('enable_special_chars'))) {
        $index++;
        continue;
      }

      // Verify if there is a special char on this class.
      if (preg_match('/[\'^£$%&*()}{@#~?><>,|=+¬]/', $attribute['attribute_key_' . $index])) {

        $url_settings_page = Url::fromRoute('block_class.settings')->toString();

        // If there is a special chat return the error for the user.
        $form_state->setErrorByName('multiple_attributes][third_party_settings][block_class][attribute_' . $index . '][attribute_key_' . $index, $this->t('Special chars is not enabled in the class value. To enable this, go to the <a href="@url_settings_page@">settings page</a>', [
          '@url_settings_page@' => $url_settings_page,
        ]));

        $form_state->setErrorByName('multiple_attributes][third_party_settings][block_class][attribute_' . $index . '][attribute_value_' . $index, $this->t('Special chars is not enabled in the class value. To enable this, go to the <a href="@url_settings_page@">settings page</a>', [
          '@url_settings_page@' => $url_settings_page,
        ]));
      }

      $index++;

    }

    $attributes = trim($attributes);

    $third_party_settings['block_class']['attributes'] = $attributes;

    // Remove the multiple_attributes attributes from ThirdPartySettings.
    unset($third_party_settings['block_class']['multiple_attributes']);

    $form_state->setValue('third_party_settings', $third_party_settings);

  }

  /**
   * Method to validate class.
   */
  public function validateClass(&$form, &$form_state, $config) {

    // If there is a settings to allow only letters and numbers, validate this.
    if (!empty($config->get('enable_special_chars'))) {
      return FALSE;
    }

    // Get the Third Party Settings from formState.
    $third_party_settings = $form_state->getValue('third_party_settings');

    if ($config->get('field_type') == 'multiple_textfields') {

      // Get the block class array from Third Party Settings.
      $multiple_items = $third_party_settings['block_class'];

      // Remove unnecessary values for this verification.
      unset($multiple_items['add_another_item']);
      unset($multiple_items['remove_item']);
      unset($multiple_items['attributes']);
      unset($multiple_items['classes']);

      // Run a foreach in all items.
      foreach ($multiple_items as $key => $item) {

        // Verify if there is a special char on this class.
        if (preg_match('/[\'^£$%&*()}{@#~?><>,|=+¬]/', $item)) {

          $url_settings_page = Url::fromRoute('block_class.settings')->toString();

          // If there is a special chat return the error for the user.
          $form_state->setErrorByName('class][third_party_settings][block_class][' . $key, $this->t('Special chars is not enabled in the class value. To enable this, go to the <a href="@url_settings_page@">settings page</a>', [
            '@url_settings_page@' => $url_settings_page,
          ]));

        }

      }

      return FALSE;

    }

    if (empty($third_party_settings['block_class']['classes'])) {
      return FALSE;
    }

    $classes = $third_party_settings['block_class']['classes'];

    if (preg_match('/[\'^£$%&*()}{@#~?><>,|=+¬]/', $classes)) {

      $form_state->setErrorByName('class][third_party_settings][block_class][classes', $this->t("In class is allowed only letters, numbers, hyphen and underline"));

    }

    $classes = explode(' ', $classes);

    foreach ($classes as $class) {

      // Validate if the first char is a point and say that isn't necessary.
      if (strpos($class, '.') === 0) {
        $form_state->setErrorByName('class][third_party_settings][block_class][classes', $this->t("Isn't necessary add point in the beginning of class"));
        return FALSE;
      }
    }

  }

}
